// Example of computing plus codes for GPS locations.
// This example uses an Adafruit GPS with Arduino (actually a Teensy).

// This example reads from the GPS in the loop() function. If your loop()
// function is not fast enough, you will need to enable interrupt-based reading.
// Look at the examples in the Adafruit_GPS library
// (https://github.com/adafruit/Adafruit_GPS).

// Licensed under the Apache License, Version 2.0 (the 'License');
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an 'AS IS' BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <Adafruit_GPS.h>

// If using hardware serial (e.g. Arduino Mega):
//   Connect the GPS TX (transmit) pin to Arduino RX1
//   Connect the GPS RX (receive) pin to matching TX1
// If using RX/TX2 or 3, change Serial1 to match your wiring:
HardwareSerial mySerial = Serial1;

// Initialise the GPS.
Adafruit_GPS gps_device(&mySerial);

// Simple timer so we can print the status every second or so.
uint32_t timer = millis();

// Echo raw GPS sentences to the serial monitor?
const bool gps_echo = true;

// Milliseconds between outputting the status.
const int update_millis = 2000;

void setup() {
  // Connect to serial at 115200 so we can read the GPS and echo without
  // dropping data.
  Serial.begin(115200);
  Serial.println("Adafruit GPS library basic test!");

  // 9600 NMEA is the default baud rate for Adafruit MTK GPS's- some use 4800
  gps_device.begin(9600);

  // Turn on RMC (recommended minimum) and GGA (fix data) including altitude.
  gps_device.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);

  // Set the update rate.
  gps_device.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);

  // Wait a second to let everything sync.
  delay(1000);

  // Ask for firmware version
  mySerial.println(PMTK_Q_RELEASE);
}

// Encode a latitude and longitude into a 10-digit (8+2) plus code.
// This is a short version of the encoding library that does not support other
// code lengths. If you need more precision or decode functionality, see
// https://github.com/google/open-location-code/
String encodePlusCode(float latitude, float longitude) {
  int a = (latitude + 90) * 1e6;
  int b = (longitude + 180) * 1e6;
  const char alpha[] = "23456789CFGHJMPQRVWX";
  String c = String("");
  for (int i = 0; i < 10; i++) {
    c = alpha[b / 125 % 20] + c;
    int d = b;
    b = a;
    a = d / 20;
  }
  return c.substring(0, 8) + '+' + c.substring(8);
}

void loop() {
  // Read data from the GPS.
  char c = gps_device.read();
  if (gps_echo && c) Serial.print(c);

  // If we have a sentence but can't parse it, just wait for the next one.
  if (gps_device.newNMEAreceived() &&
      !gps_device.parse(gps_device.lastNMEA())) {
    return;
  }

  // If millis() or timer wraps around, we'll just reset it.
  if (timer > millis()) timer = millis();

  // Approximately every 2 seconds or so, print out the current stats
  if (millis() - timer > update_millis) {
    timer = millis();  // reset the timer
    Serial.print("\nTime: ");
    Serial.print(gps_device.hour, DEC);
    Serial.print(':');
    Serial.print(gps_device.minute, DEC);
    Serial.print(':');
    Serial.print(gps_device.seconds, DEC);
    Serial.print('.');
    Serial.println(gps_device.milliseconds);
    Serial.print("Date: ");
    Serial.print(gps_device.day, DEC);
    Serial.print('/');
    Serial.print(gps_device.month, DEC);
    Serial.print("/20");
    Serial.println(gps_device.year, DEC);
    Serial.print("Fix: ");
    Serial.print(gps_device.fix);
    Serial.print(" quality: ");
    Serial.println(gps_device.fixquality);
    if (gps_device.fix) {
      Serial.print("Location: ");
      Serial.print(gps_device.latitude, 4);
      Serial.print(gps_device.lat);
      Serial.print(", ");
      Serial.print(gps_device.longitude, 4);
      Serial.println(gps_device.lon);
      Serial.print("Location (in degrees, works with Google Maps): ");
      Serial.print(gps_device.latitudeDegrees, 8);
      Serial.print(", ");
      Serial.println(gps_device.longitudeDegrees, 8);
      Serial.print("Plus code: ");
      Serial.println(encodePlusCode(gps_device.latitudeDegrees,
                                    gps_device.longitudeDegrees));

      Serial.print("Speed (knots): ");
      Serial.println(gps_device.speed);
      Serial.print("Angle: ");
      Serial.println(gps_device.angle);
      Serial.print("Altitude: ");
      Serial.println(gps_device.altitude);
      Serial.print("Satellites: ");
      Serial.println((int)gps_device.satellites);
      Serial.print("HDOP: ");
      Serial.println(gps_device.HDOP);
    }
  }
}
